#ifndef MODELSUD_H
#define MODELSUD_H

#include "kleiner-brauhelfer-core_global.h"
#include "sqltablemodel.h"

class Brauhelfer;

class LIB_EXPORT ModelSud : public SqlTableModel
{
    Q_OBJECT

    friend class ProxyModelSud;

public:

    enum Column
    {
        ColID,
        ColSudname,
        ColSudnummer,
        ColKategorie,
        ColAnlage,
        ColMenge,
        ColSW,
        ColhighGravityFaktor,
        ColFaktorHauptguss,
        ColWasserprofil,
        ColRestalkalitaetSoll,
        ColCO2,
        ColIBU,
        ColberechnungsArtHopfen,
        ColKochdauer,
        ColNachisomerisierungszeit,
        ColReifezeit,
        ColKostenWasserStrom,
        ColKommentar,
        ColStatus,
        ColBraudatum,
        ColAbfuelldatum,
        ColErstellt,
        ColGespeichert,
        Colerg_S_Gesamt,
        Colerg_W_Gesamt,
        Colerg_WHauptguss,
        Colerg_WNachguss,
        Colerg_Farbe,
        ColSWKochende,
        ColSWAnstellen,
        ColSchnellgaerprobeAktiv,
        ColSWSchnellgaerprobe,
        ColSWJungbier,
        ColTemperaturJungbier,
        ColWuerzemengeVorHopfenseihen,
        ColWuerzemengeKochende,
        ColWuerzemengeAnstellen,
        ColSpunden,
        ColSpeisemenge,
        ColJungbiermengeAbfuellen,
        Colerg_AbgefuellteBiermenge,
        Colerg_Sudhausausbeute,
        Colerg_EffektiveAusbeute,
        Colerg_Preis,
        Colerg_Alkohol,
        ColAusbeuteIgnorieren,
        ColMerklistenID,
        ColSudhausausbeute,
        ColVerdampfungsrate,
        ColVergaerungsgrad,
        ColWuerzemengeKochbeginn,
        ColSWKochbeginn,
        ColVerschneidungAbfuellen,
        ColTemperaturKarbonisierung,
        ColBemerkungBrauen,
        ColBemerkungAbfuellen,
        ColBemerkungGaerung,
        ColReifungStart,
        ColVerduennungAnstellen,
        ColBemerkungZutatenMaischen,
        ColBemerkungZutatenKochen,
        ColBemerkungZutatenGaerung,
        ColBemerkungMaischplan,
        ColBemerkungWasseraufbereitung,
        ColMengeHefestarter,
        ColSWHefestarter,
        // virtual
        ColDeleted,
        ColMengeSollAnstellen,
        ColMengeSollKochende,
        ColMengeSollKochbeginn,
        ColMengeSollVerdampfung,
        ColMengeSollHgf,
        ColSWSollAnstellen,
        ColSWSollKochende,
        ColSWSollKochbeginn,
        ColSWSollKochbeginnMitWz,
        ColSWAnteilMalz,
        ColSWAnteilZusatzMaischen,
        ColSWAnteilZusatzKochen,
        ColSWAnteilZusatzGaerung,
        ColSWAnteilHefestarter,
        ColSWAnteilZutaten,
        ColSRESoll,
        ColAlkoholSoll,
        ColSWIst,
        ColSREErwartet,
        ColSREIst,
        ColMengeIst,
        ColIbuIst,
        ColFarbeIst,
        ColCO2Ist,
        ColSpundungsdruck,
        ColGruenschlauchzeitpunkt,
        ColSpeiseNoetig,
        ColSpeiseAnteil,
        ColZuckerAnteil,
        ColWoche,
        ColReifezeitDelta,
        ColAbfuellenBereitZutaten,
        ColWuerzemengeAnstellenTotal,
        ColVerdampfungsrateIst,
        ColsEVG,
        ColtEVG,
        ColRestalkalitaetWasser,
        ColRestalkalitaetIst,
        ColPhMalz,
        ColPhMaische,
        ColPhMaischeSoll,
        ColBewertungMittel,
        // number of columns
        NumCols
    };
    Q_ENUM(Column)

public:

    ModelSud(Brauhelfer* bh, const QSqlDatabase &db = QSqlDatabase());
    void createConnections();
    QVariant dataExt(const QModelIndex &index) const Q_DECL_OVERRIDE;
    bool setDataExt(const QModelIndex &index, const QVariant &value) Q_DECL_OVERRIDE;
    Qt::ItemFlags flags(const QModelIndex &index) const Q_DECL_OVERRIDE;
    bool removeRows(int row, int count = 1, const QModelIndex &parent = QModelIndex()) Q_DECL_OVERRIDE;
    void defaultValues(QMap<int, QVariant> &values) const Q_DECL_OVERRIDE;
    QMap<int, QVariant> copyValues(int row) const Q_DECL_OVERRIDE;
    QVariant dataSud(const QVariant &sudId, int col);
    QVariant dataAnlage(int row, int col) const;
    void setDataAnlage(int row, int col, const QVariant& value);
    QVariant dataWasser(int row, int col) const;
    void setDataWasser(int row, int col, const QVariant& value);
    void update(int row, int colChanged = -1);

private slots:

    void onModelReset();
    void onRowChanged(const QModelIndex &index);
    void onOtherModelRowChanged(const QModelIndex &index);
    void onAnlageRowChanged(const QModelIndex &index);

private:

    bool setDataExt_impl(const QModelIndex &index, const QVariant &value);
    void updateSwWeitereZutaten(int row);
    void updateWasser(int row);
    void updateFarbe(int row);
    void updatePreis(int row);
    void removeRowsFrom(SqlTableModel* model, int colId, const QList<int>& sudIds);

private:

    Brauhelfer* bh;
    bool mUpdating;
    bool mSkipUpdateOnOtherModelChanged;
    QVector<double> swWzMaischenRecipe;
    QVector<double> swWzKochenRecipe;
    QVector<double> swWzGaerungRecipe;
    QVector<double> swWzGaerungCurrent;
    QVector<double> swWzUnvergaerbarRecipe;
};

#endif // MODELSUD_H
